/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
#include "Entity.h"
#include "Stage.h"
#include "BgmapManager.h"
#include "CollisionManager.h"
#include "Globals.h"
#include "MemoryPool.h"
#include "MessageDispatcher.h"
#include "Optics.h"
#include "Printing.h"
#include "Error.h"
#include "SoundManager.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// define the Sprite
__CLASS_DEFINITION(Stage, Object)


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//class's constructor
static void Stage_constructor(Stage this);

// update world's entities state
static void Stage_setEntityState(int ID, int inGameState);

// get entity's state
static inline int Stage_getEntityState(int ID);

// remove and entity from the rendering system
static void Stage_removeFromRendering(Entity entity);

// add and entity to the rendering system
static void Stage_addToRendering(Entity entity, int z);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S ATTRIBUTES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#define __STAGE_BITS_PER_ENTITY		2
#define __ENTITIESINSTAGE (__ENTITIESPERWORLD / (sizeof(WORD) << 3) * __STAGE_BITS_PER_ENTITY)

// this allows to determine which entities to load from a world definition
static WORD entityState[__ENTITIESINSTAGE];

// list of entities to render
static Entity entitiesToRender[__OBJECTLISTTAM];

// set initial world layer
int freeLayer = 31;


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// always call these to macros next to each other
__CLASS_NEW_DEFINITION(Stage, __PARAMETERS())
__CLASS_NEW_END(Stage, __ARGUMENTS(this));


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void Stage_constructor(Stage this){

	// construct base object
	__CONSTRUCT_BASE(Object,__ARGUMENTS(this));

	{
		int i = 0;
	
		// make sure the class is reset before construct the first stage
		__CALL_ONCE(Stage_resetClass);
	
		{
			// create the entity list
			this->entities = __NEW(VirtualList);
			this->removedEntities = __NEW(VirtualList);
			
			this->world = NULL;
			
			
			// set world's size
			this->size.x = 0;
			this->size.y = 0;
			this->size.z = 0;
			
			// initialize world's actor states
			for (i = 0; i < __ENTITIESINSTAGE; i++){
				
				Stage_setEntityState(i, kUnloaded);
				
			}
			
			
			// initialize world's screen's position	
			this->screenPosition.x = 0;
			this->screenPosition.y = 0;
			this->screenPosition.z = __ZZERO;
			
			// set focus actor pointer
			
			this->friction=__AIRFRICTION;
			this->floorFriction=__FLOORFRICTION;
			
			// initialize world's screen's movement state
			this->movementState.x=__PASSIVE;
			this->movementState.y=__PASSIVE;
			this->movementState.z=__PASSIVE;
			
			_screenMovementState = &this->movementState;
			
			this->selectedText = 0;
			
			// get a pointer to the collision manager
			this->collisionManager = CollisionManager_getInstance();
			
			// clear focus actor pointer
			this->focusActor = NULL;
		
			_screenPosition = &this->screenPosition;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
void Stage_destructor(Stage this){

	// remove all objects
	Stage_reset(this);
	
	// destroy the super object
	__DEALLOCATE(Object);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// process the removed entities
void Stage_processRemovedEntities(Stage this){

	VirtualNode node = VirtualList_begin(this->removedEntities);

	if(node){
		
		// destroy all entities
		for(; node ; node = VirtualNode_getNext(node)){
			
			Entity entity = (Entity)VirtualNode_getData(node);
		
			// remove from rendering here, otherwise a removed entity could
			// overwrite another entity's world layer
			Stage_removeFromRendering(entity);

			// remove from the stage's entities
			VirtualList_removeElement(this->entities, (void*)entity);
			
	 		// delete the entity
			__DELETE(entity);
		}
		
		// clear the list
		VirtualList_clear(this->removedEntities);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// reset Stage's atributes
void Stage_reset(Stage this){
	
	VirtualNode node = NULL;
	
	// first process the removed entities
	Stage_processRemovedEntities(this);
	
	// destroy all entities
	for(node = VirtualList_begin(this->entities); node ; node = VirtualNode_getNext(node)){
		
		__DELETE(VirtualNode_getData(node));
	}
	
	// clear the list
	VirtualList_clear(this->entities);
	
	this->size.x = 0;
	this->size.y = 0;
	this->size.z = 0;

	this->screenPosition.x = 0;
	this->screenPosition.y = 0;
	this->screenPosition.z = __ZZERO;

	this->focusActor = NULL;
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void Stage_resetClass(){
	
	int i = 0;
	
	for ( i = 0; i < __OBJECTLISTTAM; i++){
		entitiesToRender[i] = NULL;
	}
	
	//clear world's actor's state
	for ( i = 0; i < __ENTITIESINSTAGE; i++){
		
		Stage_setEntityState(i, kUnloaded);
	}
	
	freeLayer = 31;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// update each entity's internal state
void Stage_update(Stage this){	
	
	VirtualNode node = VirtualList_begin(this->entities);

	// update each entity
	for(; node ; node = VirtualNode_getNext(node)){
		
		// update each entity
		__VIRTUAL_CALL(void, Entity, update, (Entity)VirtualNode_getData(node));
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// render game world' entities
void Stage_render(Stage this){
	
	int i = 0;
	
	for(; (i <__OBJECTLISTTAM) && entitiesToRender[i]; i++){

		// render each entity
		__VIRTUAL_CALL(void, Entity, render, entitiesToRender[i]);		
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// copy a Stage's attributes
void Stage_copy(Stage this, Stage sourceWorld){
	
	int i = 0;
	
	Stage_reset(this);
	
	//copy world's definition
	this->world = sourceWorld->world;
	
	//copy size
	this->size = sourceWorld->size;
	
	//copy screen's position
	this->screenPosition.x = sourceWorld->screenPosition.x;
	this->screenPosition.y = sourceWorld->screenPosition.y;
	this->screenPosition.z = sourceWorld->screenPosition.z;
	
	//copy the objects
	/* Not really copy objects, just swap list's heads
	 * to make the process faster.
	 * It works because the GameEngine object
	 * has two Stages. If more Stages are defined
	 * then call the VirtualList_copy method.
	 */

	// swap actor lists
	VirtualList_swap(this->entities, sourceWorld->entities);



	//copy states
	for(i = 0; i < __ENTITIESINSTAGE; i++){
		//actorsState[i] = sourceWorld->actorsState[i];
		//actorsInGameState[i] = sourceWorld->actorsInGameState[i];
	}
	
	//copy misc attributes
	this->friction = sourceWorld->friction;
	this->bgm = sourceWorld->bgm;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// check if any entity must be assigned another world layer
void Stage_checkLayers(){

	int i = 0;

	VBVec3D position = Entity_getPosition(entitiesToRender[0]);

	CACHE_ENABLE;
	for(;i < __OBJECTLISTTAM - 1 &&  entitiesToRender[i + 1]; i++){

		VBVec3D nextPosition = Entity_getPosition(entitiesToRender[i + 1]);

		// check if z positions are swaped
		if(nextPosition.z > position.z){
			
			// get each entity's layer
			int worldLayer1 = Sprite_getWorldLayer(Entity_getSprite(entitiesToRender[i]));
			int worldLayer2 = Sprite_getWorldLayer(Entity_getSprite(entitiesToRender[i + 1]));
			
			// swap array entries
			Entity auxEntity = entitiesToRender[i];			
			entitiesToRender[i] = entitiesToRender[i + 1];			
			entitiesToRender[i + 1] = auxEntity;

			// swap layers
			Entity_setLayer(entitiesToRender[i], worldLayer1);
			Entity_setLayer(entitiesToRender[i + 1], worldLayer2);

			break;
		}
		position = nextPosition;
	}
	CACHE_DISABLE;
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
static void Stage_addToRendering(Entity entity, int z){
	
	int i = 0;
	
	for(; entitiesToRender[i] && i < __OBJECTLISTTAM; i++){
		
		// compare z positions to determine entity's placement
		if(z > Entity_getPosition(entitiesToRender[i]).z){

			int j = i;
	
			// find the last render object's index
			for(; entitiesToRender[j] && j < __OBJECTLISTTAM; j++);
			
			// copy each object to the next slot
			for(; j > i ; j--){

				entitiesToRender[j] = entitiesToRender[j - 1];
				
				// and update world layer inmediately to avoid flickering
				Entity_updateLayer(entitiesToRender[j], -1);
			}
			
			break;
		}
	}
	
	if(i < __OBJECTLISTTAM){
		
		// set entity into slot
		entitiesToRender[i] = entity;

		// set layer
		Entity_setLayer(entitiesToRender[i], 31 - i);
	}

	// reasign layers
	Stage_assignLayers();
	
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
static void Stage_removeFromRendering(Entity entity){
	
	int i = 0;
	
	CACHE_ENABLE;
	
	// search for the entity to remove
	for(; entitiesToRender[i] != entity && i < __OBJECTLISTTAM; i++);
	
	// if found
	if(i < __OBJECTLISTTAM){
		
		int j = i;
		// must render the whole entities after the entity to be removed twice
		// to avoid flickering
		for(; entitiesToRender[j + 1] && j < __OBJECTLISTTAM - 1; j++){
			
			entitiesToRender[j] = entitiesToRender[j + 1];
			
			Entity_updateLayer(entitiesToRender[j], 1);
			
		}
		
		// remove object from list
		entitiesToRender[j] = NULL;
	}
	
	CACHE_DISABLE;

	// reassign layers
	Stage_assignLayers();
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void Stage_assignLayers(){
	
	int i = 0;
	
	freeLayer = 31;
	
	for(i = 0; entitiesToRender[i] && i < __OBJECTLISTTAM; i++){
		
		//load object
		__VIRTUAL_CALL(void, Entity, setLayer, entitiesToRender[i], __VIRTUAL_ARGUMENTS(freeLayer--));
	}	

	ASSERT(freeLayer > 0, Stage: worldLayers depleted);
	
	//update printing world layer for non textboxs (use mainly for debug)	
	vbjRenderOutputText(freeLayer, BgmapManager_getFreeBgmap(BgmapManager_getInstance()));
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// create an entity in gameengine's memory  and load it in the Stage
Entity Stage_addEntity(Stage this, EntityDefinition* entityDefinition, VBVec3D position, int ID, void* extraInfo){
	
	ASSERT((int)entityDefinition->allocator, No allocator define);

	{
		// call the appropiate allocator which allows for inheritance!
		Entity entity = (Entity)((Entity (*)(EntityDefinition*, int)) entityDefinition->allocator)(entityDefinition, ID);
	
		// setup entity if allocated and constructed
		if(entity){
	
			// set spatial position
			__VIRTUAL_CALL(void, Entity, setPosition, entity, __VIRTUAL_ARGUMENTS(position));
			
			// process extra info
			if(extraInfo){
				
				__VIRTUAL_CALL(void, Entity, setExtraInfo, entity, __VIRTUAL_ARGUMENTS(extraInfo));
			}
			
			// assign the entity a layer to be displayed
			Stage_addToRendering(entity, position.z);
	
			// put the background in out list
			VirtualList_pushFront(this->entities, (void*)entity);
			
			return entity;
		}
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// remove an entity from the game
void Stage_removeEntity(Stage this, Entity entity, int inGameState){
	
	// remove from the list of objects
	VirtualList_pushFront(this->removedEntities, (void*)entity);
	
	// if state is loaded... just set it as unloaded
	if(kLoaded == inGameState){
		
		inGameState = kUnloaded;
	}
	
	// update in game state
	Stage_setEntityState(Entity_getID(entity), inGameState);
	
	// clear if it is the focus actor
	if((Entity)this->focusActor == entity){
		
		this->focusActor = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// update world's entities state
static void Stage_setEntityState(int ID, int inGameState){
	
	int bitIndex =  ID * __STAGE_BITS_PER_ENTITY;
	
	// arrayIndex = bitIndex / (sizeof(WORD) << 3);
	int arrayIndex = bitIndex >> (sizeof(WORD) + 1);
	
	// displacement =  bitIndex % (sizeof(WORD) << 3);
	int displacement =  bitIndex & ((sizeof(WORD) << 3) - 1);

	WORD mask = entityState[arrayIndex] & vbRotate(0xFFFFFFFF << __STAGE_BITS_PER_ENTITY, displacement, __ROT_LEFT);
	
	// make sure we are unloading an entity loaded by 
	// me and not but the programmer
	if(0 > ID){
		
		return;
	}
	
	mask |=  ((WORD)inGameState << displacement);

	entityState[arrayIndex] = mask;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// get entity's state
static inline int Stage_getEntityState(int ID){
	
	int bitIndex =  ID * __STAGE_BITS_PER_ENTITY;
	
	// arrayIndex = bitIndex / (sizeof(WORD) << 3);
	int arrayIndex = bitIndex >> (sizeof(WORD) + 1);
	
	// displacement =  bitIndex % (sizeof(WORD) << 3);
	int displacement =  bitIndex & ((sizeof(WORD) << 3) - 1);

	return (int)(entityState[arrayIndex] >> displacement) & 0x03;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// place holder for objects designed around OBJECTS in the VB hardware
void Stage_setupObjActor(int *actor,int x,int y, int z){

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// load stage's entites
void Stage_load(Stage this, StageDefinition* world){
	
	int i = 0;
	
	//stop all sounds
	SoundManager_stopAllSound(SoundManager_getInstance());
	
	// TODO
	//Stage_constructor(this);	
	CollisionManager_reset(this->collisionManager);
	
	//set world's definition
	this->world = world;
		
	
	//this->number=worldNumber;
	//set world's limits		
	this->size = world->size;
	
	//_Stage.start=_Stage.end=NULL;
	
	//initial screen position within the world
	this->screenPosition = world->position;
	
	//load Stage's bgm
	this->bgm = (u16 (*)[6])world->bgm;
	
	// load entities
	for(; world->entities[i].entity; i++){
		
		// test if the position is inside the game
		EntityDefinition* entityDefinition = world->entities[i].entity;

		VBVec3D position = {
				ITOFIX23_9(world->entities[i].position.x),
				ITOFIX23_9(world->entities[i].position.y),
				ITOFIX23_9(world->entities[i].position.z)
				};

		// if entity is visible
		if(vbjIsVisible(position, 
				entityDefinition->spriteDefinition.textureDefinition->cols << 3, 
				entityDefinition->spriteDefinition.textureDefinition->rows << 3,
				0,
				__ENTITYLOADPAD
				)){

			//load actor in world			
			Stage_addEntity(this, world->entities[i].entity, position, i, world->entities[i].extraInfo);
				
			Stage_setEntityState(i, kLoaded);
		}		
		else{
			//otherwise set object as alive but not loaded yet
			Stage_setEntityState(i, kUnloaded);
		}
	}
	
	//load background music
	SoundManager_loadBGM(SoundManager_getInstance(),(u16 (*)[6])this->bgm);
	
	//setup the column table
	vbSetColTable();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// add the object's address from a source world
void Stage_addWorldObjectsReferences(Stage this, Stage source){
	
	VirtualNode node = NULL;
	
	// add actors
	for(node = VirtualList_begin(source->entities); node ; node = VirtualNode_getNext(node)){
		
		VirtualList_pushFront(this->entities, (void*) VirtualNode_getData(node));
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// remove the object's address from a source world
void Stage_removeWorldObjectsReferences(Stage this, Stage source){
	
	VirtualNode node = NULL;
	
	// add actors
	for(node = VirtualList_begin(source->entities); node ; node = VirtualNode_getNext(node)){
		
		VirtualList_removeElement(this->entities, (void*) VirtualNode_getData(node));
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// get actual free layer
int Stage_getFreeLayer(){
	
	return freeLayer;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// write in graphic memory the entities within the stage
void Stage_writeEntities(Stage this){

	VirtualNode node = NULL;
	
	for(node = VirtualList_begin(this->entities); node ; node = VirtualNode_getNext(node)){
		
		//Entity_resetMemoryState((Actor)VirtualNode_getData(node), freeLayer--);		
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// retrieve stage size
Size Stage_getSize(Stage this){
	
	return this->size;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// determine if a point is visible
inline static int Stage_inLoadRange(Stage this, VBVec3D* const position, int width, int height){
	
	int xLowLimit = -(width) ;
	int xHighLimit = __SCREENWIDTH + (width);

	int yLowLimit = -(height) ;
	int yHighLimit = __SCREENHEIGHT + (height);

	VBVec2D position2D;
	
	VBVec3D position3D = vbjNormalizePosition(position);
	
	
	//project the position to 2d space
	vbjProjectFastTo2D(&position2D, &position3D);

	//(x >= min && x < max) can be transformed into (unsigned)(x-min) < (max-min)
	// check x visibility
	if((unsigned)(position2D.x - xLowLimit) < (position2D.x - xHighLimit)){		
		
		// check y visibility
		if((unsigned)(position2D.y - yLowLimit) < (position2D.y - yHighLimit)){
		
			// check z visibility
			//if(position3D.z >= _screenPosition->z && position3D.z < _screenPosition->z + ITOFIX23_9(this->size.z)){
				
				
				return false;
			//}
		}	
	}
	
	xLowLimit -= __ENTITYLOADPAD;
	xHighLimit += __ENTITYLOADPAD;
	yLowLimit -= __ENTITYLOADPAD;
	yHighLimit += __ENTITYLOADPAD;	
	
	// check x visibility
	if((unsigned)(position2D.x - xLowLimit) < (position2D.x - xHighLimit)){
		
		// check y visibility
		if((unsigned)(position2D.y - yLowLimit) < (position2D.y - yHighLimit)){
		
			// check z visibility
			//if(position3D.z >= _screenPosition->z && position3D.z < _screenPosition->z + ITOFIX23_9(this->size.z)){
				
				return true;
			//}
		}	
	}	
	
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// load entities on demand (if they aren't loaded and are visible)
void Stage_loadEntities(Stage this){

	StageDefinition* world = this->world;
	
	int i = 0;
	
	// TODO: don't check every entity in the stage's definition
	// must implement an algorithm which only check a portion of it
	
	//go through n entities in Stage	
	for(; i < __ENTITIESPERWORLD && world->entities[i].entity; i++){
		
		//if entity isn't loaded and haven't been killed
		int inGameState = Stage_getEntityState(i);		
		if(kLoaded != inGameState && kDead != inGameState){
			
			// test if the position is inside the game
			EntityDefinition* entityDefinition = world->entities[i].entity;
			
			VBVec3D position = {
					ITOFIX23_9(world->entities[i].position.x),
					ITOFIX23_9(world->entities[i].position.y),
					ITOFIX23_9(world->entities[i].position.z)
					};
			
			// if entity in load range
			if(Stage_inLoadRange(this, &position, 
					entityDefinition->spriteDefinition.textureDefinition->cols << 2, 
					entityDefinition->spriteDefinition.textureDefinition->rows << 2)){
				
				// create the entity and add it to the world
				Stage_addEntity(this, world->entities[i].entity, position, i, world->entities[i].extraInfo);
		
				if(kUnloaded == inGameState){
					 
					inGameState = kLoaded;
				}
				
				//set actor state as loaded
				Stage_setEntityState(i, inGameState);
				
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// unload non visible entities
void Stage_unloadEntities(Stage this){
	
	VirtualNode node = VirtualList_begin(this->entities);

	CACHE_ENABLE;
	// check which actors must be unloaded
	for(; node ; node = VirtualNode_getNext(node)){
		
		//load actor
		Entity entity = (Entity)VirtualNode_getData(node);
		
		//if actor isn't visible inside the view field, unload it
		if(!__VIRTUAL_CALL(int, Entity, isVisible, entity, __VIRTUAL_ARGUMENTS(__ENTITYLOADPAD))){		

			// remove from the list
			Stage_removeEntity(this, entity, __VIRTUAL_CALL(int, Entity, getInGameState, entity));
		}
	}
	CACHE_DISABLE;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// center world's screen in function of focus actor's position
void Stage_centerScreen(Stage this){

	//if focusactor is defined
	if(this->focusActor){
		
		//get focusactor's movement state
		MovementState movementState = Actor_getMovementState(this->focusActor);
		
		if(movementState.x){
			
			//get focusactor's position
			VBVec3D focusPosition = Entity_getPosition((Entity)this->focusActor);
			
			//set world's screen's movement
			this->movementState = movementState;
			
			// if screen isn't at the left limit
			// set it's position in function of
			// focusactor's position
			 
			this->screenPosition.x = 0;
	
			if(focusPosition.x - ITOFIX23_9(192) >= 0){
				
				this->screenPosition.x = focusPosition.x - ITOFIX23_9(192);
				
				if(focusPosition.x + ITOFIX23_9(192) > ITOFIX23_9(this->world->size.x)){
					
					this->screenPosition.x = ITOFIX23_9(this->world->size.x - 384);
					
				}
			}
		}
		else{
			
			//stop world's screen's movement
			this->movementState.x = __PASSIVE;
			this->movementState.y = __PASSIVE;
			this->movementState.z = __PASSIVE;

		}
	}
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// get stage's friction attribure
float Stage_getFriction(Stage this){
	
	return this->friction;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
float Stage_getFloorFriction(Stage this){
	
	return this->floorFriction;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// get stage's friction attribure
void Stage_setFriction(Stage this, fix7_9 friction){

	this->friction=friction;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void Stage_setFloorFriction(Stage this, fix7_9 floorFriction){
	
	this->floorFriction = floorFriction;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BGMPointer Stage_getBGM(Stage this){
//int ((*)[]) Stage_getBGM(Stage this){	
	return this->bgm;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// process a key pressed
int Stage_onKeyPressed(Stage this){

	VirtualNode node = VirtualList_begin(this->entities);
	
	// tell each actor about the key pressed
	for(; node; node = VirtualNode_getNext(node)){
		
		if(MessageDispatcher_dispatchMessage(0, NULL, 
				(Entity)VirtualNode_getData(node), kKeyPressed, NULL)){
			
			return true;
		}
	}
	
	return false;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// remove all entities from the stage
void Stage_cleanUp(Stage this){
	
	// clean myself
	Stage_reset(this);
	
	// clean class's attibutes
	Stage_resetClass();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set the focus entity
void Stage_setFocusActor(Stage this, Actor focusActor){
	
	this->focusActor = focusActor;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// retrieve focus entity
Actor Stage_getFocusActor(Stage this){
	
	return this->focusActor;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// retrieve a layer
void Stage_giveLayer(){
	
	freeLayer--;
}