/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef STAGE_H_
#define STAGE_H_


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include <libgccvb.h>
#include "Object.h"
//#include "Constants.h"
#include "MiscStructs.h"
#include "Entity.h"
#include "Actor.h"
#include "VirtualList.h"




/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// declare the virtual methods
#define Stage_METHODS								\
	Object_METHODS;									\
	__VIRTUAL_METHOD(update);						\
	__VIRTUAL_METHOD(render);

// declare the virtual methods which are redefined
#define Stage_SET_VTABLE(ClassName)				\
	Object_SET_VTABLE(ClassName)				\
	__VIRTUAL_SET(ClassName,Stage,update);		\
	__VIRTUAL_SET(ClassName,Stage,render);			
	

// declare a Stage, which holds the objects in a game world
__CLASS(Stage);


#define Stage_ATTRIBUTES						\
												\
	/* it is derivated from*/					\
	Object_ATTRIBUTES							\
												\
	/* world's definition pointer */			\
	StageDefinition* world;						\
												\
	/* world size in pixels */					\
	Size size;									\
												\
	/* screen position */						\
	VBVec3D screenPosition;						\
												\
	/* list of Stage's entities */				\
	VirtualList entities;						\
												\
	/* list of entities to remove */			\
	VirtualList removedEntities;				\
												\
	/* actual active interactive text */		\
	int selectedText;							\
												\
	/* actor to center the screen around */		\
	Actor focusActor;							\
												\
	/* world's friction */						\
	fix7_9 friction;							\
												\
	/* friction factor for world's floor */		\
	float floorFriction;						\
												\
	/* world's screen's movement state */		\
	MovementState movementState;				\
												\
	/* pointer to background music */			\
	u16 (*bgm)[6];								\
												\
	/* a pointer to the collision manager */	\
	CollisionManager collisionManager;			


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S ROM DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// a actor asociated with a position
typedef const struct PositionedEntity{
	
	// pointer to the entity definition in ROM
	EntityDefinition* entity;
	
	// position in the world
	VBVec3DReal position;

	// extra info
	void* extraInfo;
	
}PositionedEntity;

/* ---------------------------------------------------------------------------------------------------------*/
// defines a game world in ROM memory
typedef const struct  StageDefinition{
	
	// world's size over each axis in pixels
	Size size;
	
	// initial screen's position inside the game world
	VBVec3D position;
	
	// pointer to the background music
	u16 (*bgm)[];

	// each of the stage's entities
	PositionedEntity entities[__ENTITIESPERWORLD];

}StageDefinition;
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// allocate memory and call the constructor
Stage Stage_new();

// class's destructo
void Stage_destructor(Stage this);

// give each entity a world layer to be rendered
void Stage_assignLayers();

// delete from memory the removed entities
void Stage_processRemovedEntities(Stage this);

// reset Stage's atributes
void Stage_reset(Stage this);

// reset class's static members
void Stage_resetClass();

// refreshes each actor defined
void Stage_update(Stage this);

// render game world' entities
void Stage_render(Stage this);

// copy a Stage's attributes
void Stage_copy(Stage this, Stage sourceWorld);

// check if any entity must be assigned another world layer
void Stage_checkLayers();

// add an entity to the stage
Entity Stage_addEntity(Stage this, EntityDefinition* entityDefinition, VBVec3D position, int ID, void* extraInfo);

// remove an entity from the game
void Stage_removeEntity(Stage this, Entity entity, int inGameState);

// load stage's entites
void Stage_load(Stage this, StageDefinition* world);

// add the object's address from a source world
void Stage_addWorldObjectsReferences(Stage this, Stage source);

// remove the object's address from a source world
void Stage_removeWorldObjectsReferences(Stage this, Stage source);

// get actual free layer
int Stage_getFreeLayer();

// write in graphic memory the actors within the world
void Stage_writeEntities(Stage this);

// write in graphic memory the textbox within the world
void Stage_writeTextBoxList(Stage this);

// write in graphic memory the backgrounds within the world
void Stage_writeBackgroudList(Stage this);

// write in graphic memory the scrolls within the world
void Stage_writeScrollList(Stage this);

// retrieve stage's size
Size Stage_getSize(Stage this);

// load objects on demand (if they aren't loaded and are visible)
void Stage_loadEntities(Stage this);

// unload non visible entities
void Stage_unloadEntities(Stage this);

// center stage's screen in function of focus actor's postion
void Stage_centerScreen(Stage this);

// get stage's friction attribure
float Stage_getFriction(Stage this);

float Stage_getFloorFriction(Stage this);

// get stage's friction attribure
void Stage_setFriction(Stage this, fix7_9 friction);

void Stage_setFloorFriction(Stage this, fix7_9 floorFriction);

// print stage's objects states
void Stage_printStates(Stage this, int x, int y, int list);

typedef u16 (*BGMPointer)[];
BGMPointer Stage_getBGM(Stage this);

// get textbox list
VirtualList Stage_getTextBoxList(Stage this);

// process a key pressed
int Stage_onKeyPressed(Stage this);

// remove all entities from the stage
void Stage_cleanUp(Stage this);

// set the focus entity
void Stage_setFocusActor(Stage this, Actor focusActor);

// retrieve focus entity
Actor Stage_getFocusActor(Stage this);

// retrieve a layer
void Stage_giveLayer();

#endif

